/****************************************************************
 *  Kommute is distributed under the following license:
 *
 *  Copyright (C) 2008, defnax
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 *  Boston, MA  02110-1301, USA.
 ****************************************************************/

//#include "StatisticPage.h"
//#include <control/bandwidthevent.h>
#include "kommute.h"
#include "linetypes.h"
#include <QHeaderView>

/** Redraw graph every 1000ms **/
#define REFRESH_RATE    1000

/** Constructor */
void Kommute::initStatisticPage()
{

   /* Create Bandwidth Graph related QObjects */
  
   /* Set header resize modes and initial section sizes */
   QHeaderView * _stheader = statsTreeWidget-> header();
   _stheader->resizeSection ( 0, 210 );

  /* Bind events to actions */
  createActions();

  /* Ask Mute to notify us about bandwidth updates */


  /* Initialize Sent/Receive data counters */
  reset();
  
  /* Hide Bandwidth Graph Settings frame */
  showSettingsFrame(false);
  showUploadsFrame(false);

  /* Turn off opacity group on unsupported platforms */
#if defined(Q_WS_WIN)
  if(!(QSysInfo::WV_2000 <= QSysInfo::WindowsVersion <= QSysInfo::WV_2003)) {
    frmOpacity->setVisible(false);
  }
#endif
  
#if defined(Q_WS_X11)
  frmOpacity->setVisible(false);
#endif
}

/** Default destructor 
StatisticPage::~StatisticPage()
{
  //delete _settings;
}*/

/**
 Custom event handler. Checks if the event is a bandwidth update event. If it
 is, it will add the data point to the history and updates the graph.

void
StatisticPage::customEvent(QEvent *event)
{
  if (event->type() == CustomEventType::BandwidthEvent) {
    BandwidthEvent *bw = (BandwidthEvent *)event;
    updateGraph(bw->bytesRead(), bw->bytesWritten());
  }
}*/

/**
 Binds events to actions
*/
void
Kommute::createActions()
{
  connect(btnToggleSettings, SIGNAL(toggled(bool)),
      this, SLOT(showSettingsFrame(bool)));

  connect(btnReset, SIGNAL(clicked()),
      this, SLOT(reset()));

  connect(btnSaveSettings, SIGNAL(clicked()),
      this, SLOT(saveChanges()));

  connect(btnCancelSettings, SIGNAL(clicked()),
      this, SLOT(cancelChanges()));
      
  connect(btnToggleSettings2, SIGNAL(toggled(bool)),
      this, SLOT(showUploadsFrame(bool)));
      
 connect(btnReset2, SIGNAL(clicked()),
      this, SLOT(reset()));
      
 connect(btnSaveSettings2, SIGNAL(clicked()),
      this, SLOT(saveChanges()));
      
 connect(btnCancelSettings2, SIGNAL(clicked()),
      this, SLOT(cancelChanges()));
  
  
  //connect(ui.sldrOpacity, SIGNAL(valueChanged(int)),
  //    this, SLOT(setOpacity(int)));
}

/**
 Adds new data to the graph
*/
void
Kommute::updateGraph(quint64 bytesRead, quint64 bytesWritten)
{
  /* Graph only cares about kilobytes */
  frmGraph->addPoints(bytesRead/1024.0, bytesWritten/1024.0);
  
  /* Graph only cares about kilobytes */
  frmGraph2->addPoints(bytesRead/1024.0, bytesWritten/1024.0);
}

/**
 Loads the saved Bandwidth Graph settings
*/
void
Kommute::loadSettings()
{
  /* Set window opacity slider widget */
  //ui.sldrOpacity->setValue(_settings->getBWGraphOpacity());
  


  /* Set the line filter checkboxes accordingly */
  //uint filter = _settings->getBWGraphFilter();
  //ui.chkReceiveRate->setChecked(filter & BWGRAPH_REC);
  //ui.chkSendRate->setChecked(filter & BWGRAPH_SEND);

  /* Set graph frame settings */
  frmGraph->setShowCounters(chkReceiveRate->isChecked(),
                               chkSendRate->isChecked());
}

/** 
 Resets the log start time
*/
void
Kommute::reset()
{
  /* Set to current time */
//  ui.statusbar->showMessage(tr("Since:") + " " + 
//			    QDateTime::currentDateTime()
//			    .toString(DATETIME_FMT));
  /* Reset the graph */
  frmGraph->resetGraph();
  frmGraph2->resetGraph();
}

/**
 Saves the Bandwidth Graph settings and adjusts the graph if necessary
*/
void
Kommute::saveChanges()
{
  /* Hide the settings frame and reset toggle button */
  showSettingsFrame(false);
  showUploadsFrame(false);
  
  /* Save the opacity */



  
  /* Save the line filter values */
  //_settings->setBWGraphFilter(BWGRAPH_REC, ui.chkReceiveRate->isChecked());
  //_settings->setBWGraphFilter(BWGRAPH_SEND, ui.chkSendRate->isChecked());

  /* Update the graph frame settings */
  frmGraph->setShowCounters(chkReceiveRate->isChecked(),
                               chkSendRate->isChecked());
                               
    /* Update the graph frame settings */
  frmGraph2->setShowCounters(chkReceiveRate2->isChecked(),
                               chkSendRate2->isChecked());

  /* A change in window flags causes the window to disappear, so make sure
   * it's still visible. */
  showNormal();
}

/** 
 Simply restores the previously saved settings
*/
void 
Kommute::cancelChanges()
{
  /* Hide the settings frame and reset toggle button */
  showSettingsFrame(false);
  showUploadsFrame(false);

  /* Reload the settings */
  //loadSettings();
}

/** 
 Toggles the Settings pane on and off, changes toggle button text
*/
void
Kommute::showSettingsFrame(bool show)
{
  if (show) {
    frmSettings->setVisible(true);
    btnToggleSettings->setChecked(true);
    btnToggleSettings->setText(tr("Hide Settings"));
  } else {
    frmSettings->setVisible(false);
    btnToggleSettings->setChecked(false);
    btnToggleSettings->setText(tr("Show Settings"));
  }
}

void
Kommute::showUploadsFrame(bool show)
{
  if (show) {
    frmUploads->setVisible(true);
    btnToggleSettings2->setChecked(true);
    btnToggleSettings2->setText(tr("Hide Settings"));
  } else {
    frmUploads->setVisible(false);
    btnToggleSettings2->setChecked(false);
    btnToggleSettings2->setText(tr("Show Settings"));
  }
}

/**
 Sets the opacity of the Bandwidth Graph window
*/


/** 
 Overloads the default show() slot so we can set opacity

void
StatisticPage::show()
{
  loadSettings();
  if(!this->isVisible()) {
    QMainWindow::show();
  } else {
    QMainWindow::activateWindow();
    setWindowState(windowState() & ~Qt::WindowMinimized | Qt::WindowActive);
    QMainWindow::raise();
  }
}
*/

