/***************************************************************************
 *   Copyright (C) 2005 by Niels Sandholt Busch   *
 *   niels@diku.dk   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DOWNLOAD_H
#define DOWNLOAD_H

#include <QString>
#include <QObject>
#include <QPersistentModelIndex>
#include <QTime>
#include "xprogressbar.h"


class QEvent;
class QStandardItemModel;
class xProgressBar;


namespace DownloadQueue {
// An item in the download queue can be in four different states related to the queue:
// Delayed, Queued, Active, Inactive
enum state { DELAYED, QUEUED, ACTIVE, INACTIVE };
}

/**
@author Niels Sandholt Busch
*/
class Download : public QObject
{
Q_OBJECT
public:
    Download( QStandardItemModel* model, QString fname, unsigned long fsize, QString fhash, QString fhost);

    ~Download();

    // Receives events from the download thread
    void customEvent( QEvent *e );
    // Starts the download
    void start();
    // Cancel the download
    void cancel();
    // Force download
    void forceDequeue();
    // Get the download status
    DownloadQueue::state state();
    // Get the fileName
    QString fileName();
    // Get the host
    QString host();
    // Delete hash file
    void delHash();
    unsigned long getSize(){return size;};
    QString getHash(){return hash;};
    
    void setRowColor(int c, QColor color);


signals:
    // Signals the item is not in state ACTIVE anymore
//     void deactivated( QTreeWidgetItem* );
    // Signals a download started
    void downloadStarted( QString );
    // Signals a download ended - for any reason ( complete, failed, canceled )
    void downloadEnded( QString );

private:
    void setName( QString );
    void setHost( QString );
    void setProgress( int, unsigned long );
    void setStatus( int );
    void setRouteQuality( int );
    void setSources( int );
    void updateProgress();
    void writeHash();
    int myRow();
    

    class DownloadThread *dlThread;
    QString name, hash, m_host;
    unsigned long size;
    DownloadQueue::state queueState;
    QStandardItemModel *model;
    //QPersistentModelIndex nameIndex, progressIndex, hostIndex, statusIndex, routeIndex, sourcesIndex;
    int routeQuality, currentSize ;
    int currentStatus;
    QTime blockStartTime[10];
    int blockStartSize[10];
    int nbStart;
    int currentRate;
};

#endif
