/***************************************************************************
 *   Copyright (C) 2006 by Niels Sandholt Busch                            *
 *   niels@diku.dk                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "downloadstable.h"
#include "download.h"
#include "DLListDelegate.h"

#include <QContextMenuEvent>
#include <QMenu>
#include <QAction>
#include <QTableView>
#include <QTreeView>
#include <QHeaderView>

DownloadsTable::DownloadsTable (QWidget * parent):QTreeView (parent)
{
  //verticalHeader ()->hide ();
}

DownloadsTable::~DownloadsTable ()
{
}

void
DownloadsTable::contextMenuEvent (QContextMenuEvent * e)
{
  QMenu popMenu (this);
  QAction *actionDownload = popMenu.addAction (tr ("Resume"));
  actionDownload->setIcon (QIcon (":/resources/start.png"));
  connect (actionDownload, SIGNAL (triggered ()), this, SLOT (download ()));
  
  QAction *actionCancel = popMenu.addAction (tr ("Cancel"));
  actionCancel->setIcon (QIcon (":/delete.png"));
  connect (actionCancel, SIGNAL (triggered ()), this, SLOT (cancel ()));
  
  popMenu.addSeparator ();
  
  //QAction *actionDetails = popMenu.addAction (tr ("Details..."));
  //actionDetails->setIcon (QIcon (":/resorces/fileinfo.png"));
  //connect (actionDetails, SIGNAL (triggered ()), this, SLOT (details ()));
  
  //popMenu.addSeparator ();
  
  QAction *actionClear = popMenu.addAction (tr ("Clear"));
  actionClear->setIcon (QIcon (":/deleteall.png"));
  connect (actionClear, SIGNAL (triggered ()), this, SLOT (clear ()));
  
  popMenu.addSeparator ();
  
  QAction *actionGrab = popMenu.addAction (tr ("Grab Friend"));
  actionGrab->setIcon (QIcon (":/grabfriend.png"));
  connect (actionGrab, SIGNAL (triggered ()), this, SLOT (grab ()));

  popMenu.setMouseTracking (true);
  popMenu.exec (QCursor::pos ());
}

void
DownloadsTable::cancelAll ()
{
  selectAll();
  QModelIndexList lst = selectedIndexes ();
  for (int i = 0; i < lst.count (); i++)
    {
      if (lst[i].column () == 0)
	{
          QModelIndex & ind = lst[i];
	  Download *dl =
	    (Download *) (ind.model ()->
			  data (ind.model ()->index (ind.row (), PTR)).value <
			  quintptr > ());
	  dl->cancel ();
	}
    }
}

void
DownloadsTable::cancel ()
{
  QModelIndexList lst = selectedIndexes ();
  for (int i = 0; i < lst.count (); i++)
    {
      if (lst[i].column () == 0)
	{
          QModelIndex & ind = lst[i];
	  Download *dl =
	    (Download *) (ind.model ()->
			  data (ind.model ()->index (ind.row (), PTR)).value <
			  quintptr > ());
	  dl->cancel ();
	  dl->delHash ();
	}
    }
}

void
DownloadsTable::download ()
{
  QModelIndexList lst = selectedIndexes ();
  for (int i = 0; i < lst.count (); i++)
    {
      if (lst[i].column () == 0)
	{
          QModelIndex & ind = lst[i];
	  Download *dl =
	    (Download *) (ind.model ()->
			  data (ind.model ()->index (ind.row (), PTR)).value <
			  quintptr > ());
printf("download %d:%s\n",i,dl->fileName().toLocal8Bit().data());
	  dl->forceDequeue ();
	}
    }
}

void
DownloadsTable::clear ()
{
  QModelIndexList lst = selectedIndexes ();
  for (int i = 0; i < lst.count (); i++)
    {
      if (lst[i].column () == 0)
	{
          QModelIndex & ind = lst[i];
	  Download *dl =
	    (Download *) (ind.model ()->
			  data (ind.model ()->index (ind.row (), PTR)).
			  value < quintptr > ());
	  dl->cancel ();
	  dl->delHash ();
	  model ()->removeRow (ind.row ());
	}
    }
}

void
DownloadsTable::grab ()
{
  QModelIndexList lst = selectedIndexes ();
  for (int i = 0; i < lst.count (); i++)
    {
      if (lst[i].column () == 0)
	{
          QModelIndex & ind = lst[i];
	  Download *dl =
	    (Download *) (ind.model ()->
			  data (ind.model ()->index (ind.row (), PTR)).
			  value < quintptr > ());
	  QString fname = dl->fileName ();
	  unsigned long fsize = dl->getSize ();
	  QString fhash = dl->getHash ();
	  QString fhost = dl->host ();
	  emit grabFriendRequested (fname, fsize, fhash, fhost);
	}
    }
}
