/***************************************************************************
 *   Copyright (C) 2007 kommute dev team                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "hashthread.h"
#include "mutelayer.h"
#include <QApplication>
#include <QTime>
#include <QFile>
#include <QFileInfo>
#include <QDir>
#include <QDirIterator>
#include <QTextStream>
#include <QTextCodec>
#include <QCryptographicHash>
#include "MUTE/fileShare.h"
#include "minorGems/util/SettingsManager.h"
#include "minorGems/formats/encodingUtils.h"
#include "sharedfilesevent.h"

extern char *muteInternalDirectoryTag;
char *muteShare_internalFileToEncodedPath (File * inFile);

SHASH *Shares = NULL;
int maxShares = 0;
int nbShares = 0;

void
HashThread::loadHash ()
{
  printf ("loading hashes\n");
  Shares = (SHASH *) malloc (5001 * sizeof (SHASH));
  memset (Shares, 0, 5001 * sizeof (SHASH));
  maxShares = 5000;
  char *sharePathString = MuteLayer::muteLayerGetSharingPath ();
  QString fullName (sharePathString);
  fullName += "/";
  fullName += muteInternalDirectoryTag;
  fullName += "/hashes";
  QFile ficin (fullName);
  if (!ficin.open (QIODevice::ReadOnly))
    return;
  QTextStream Stream (&ficin);
  Stream.setCodec ("UTF-8");
  QString n, h, s, t;
  while (!Stream.atEnd ())
    {
      n = Stream.readLine ();
      h = Stream.readLine ();
      s = Stream.readLine ();
      t = Stream.readLine ();
      fullName = QString (sharePathString) + "/" + n;
      QFileInfo fInfo (fullName);
      if (fInfo.exists ())
	{
	  Shares[nbShares].name = strdup (n.toLocal8Bit ().data ());
	  Shares[nbShares].hash = strdup (h.toLocal8Bit ().data ());
	  Shares[nbShares].size = s.toLongLong ();
	  Shares[nbShares].time = new QDateTime ();
	  (*Shares[nbShares].time) =
	    Shares[nbShares].time->fromString (t, Qt::ISODate);
	  if (fInfo.size () == Shares[nbShares].size
	      && fInfo.lastModified () == *Shares[nbShares].time)
	    {
	      doPost (&Shares[nbShares]);
	      nbShares++;
	      if (nbShares >= maxShares)
		{
		  maxShares += 1000;
		  Shares =
		    (SHASH *) realloc (Shares,
				       (maxShares + 1) * sizeof (SHASH));
		}
	    }
	  else
	    printf ("hash not ok for %s\n", Shares[nbShares].name);
	}
      else
	printf ("file not found %s\n", n.toLocal8Bit ().data ());
    }
  delete[]sharePathString;
  printf ("%d hashes loaded\n", nbShares);
}

void
HashThread::doPost (SHASH * share)
{
  QTextCodec *codecutf = QTextCodec::codecForName ("utf8");
  QTextCodec *codeclat = QTextCodec::codecForName ("iso8859-1");
  QString qstr;
  // try to convert string from utf8
  qstr = codecutf->toUnicode (share->name);
  // test if conversion is reversible
  if (strcmp (share->name, qstr.toUtf8 ().data ()))
    // if not reversible, it's not utf8, convert from iso8859 :
    qstr = codeclat->toUnicode (share->name);
  QApplication::postEvent (eventReceiver,
			   new SharedFilesEvent (qstr,
						 share->size, share->hash));
}

void
saveHash ()
{
  printf ("saving %d hashes\n", nbShares);
  char *sharePathString = MuteLayer::muteLayerGetSharingPath ();
  QString fullName (sharePathString);
  fullName += "/";
  fullName += muteInternalDirectoryTag;
  fullName += "/hashes.new";
  QFile qficout (fullName);
  if (!qficout.open (QIODevice::WriteOnly))
    return;
  QTextStream Stream (&qficout);
  for (int i = 0; i < nbShares; i++)
    {
      // use write to avoid conversion from iso8859 made by qstream
      qficout.write (Shares[i].name, strlen (Shares[i].name));
      Stream << endl << Shares[i].hash << endl;
      Stream << Shares[i].size << endl;
      Stream << Shares[i].time->toString (Qt::ISODate) << endl;
    }
  qficout.close ();
  fullName =
    QString (sharePathString) + "/" + QString (muteInternalDirectoryTag) +
    "/hashes";
  qficout.remove (fullName);
  qficout.rename (fullName);
  delete[]sharePathString;
  printf ("hashes saved\n");
}

int
findFilename (char *filename)
{
  for (int i = 0; i < nbShares; i++)
    if (!strcmp (filename, Shares[i].name))
      return i;
  return -1;
}


int percentCPU = 25;


HashThread::HashThread (QObject * eventReceiver):QThread (), eventReceiver (eventReceiver),
running (false)
{
  time0 = QTime::currentTime ();
}

HashThread::~HashThread ()
{
  wait ();
  if(Shares)
  {
    free(Shares);
    Shares=NULL;
  }
}

void
HashThread::run ()
{
  running = true;
  loadHash ();
  while (running)
    {
      doHash ();
      // sleep half an hour before retry hashing
      printf ("hashes computed, wait half an hour...\n");
      int cpt=3600000;
      while (running && cpt-- > 0)
	usleep (500);
    }
}


void
HashThread::stop ()
{
  running = false;
  saveHash ();
}

void
HashThread::doHash ()
{
  char *sharedPath = muteShareGetSharingPath ();
  QDir sharedDirectory (sharedPath);
  delete[]sharedPath;
  if (sharedDirectory.exists ())
    {
      // make sure hash dir exists
      QFileInfo hashDirectory (sharedDirectory, muteInternalDirectoryTag);
      if (!hashDirectory.exists ())
	{
	  sharedDirectory.mkdir (muteInternalDirectoryTag);
	}
      // look at child files recursively
      char found;
      int maxDepth = SettingsManager::getIntSetting ("maxSubfolderDepth",
						     &found);
      if (!found || maxDepth < 0)
	{
	  // default max depth of 10
	  maxDepth = 10;
	}
      QDirIterator it (sharedDirectory,
		       QDirIterator::Subdirectories | QDirIterator::
		       FollowSymlinks);
      while (it.hasNext () && running)
	{
	  QString nextname = it.next ();
	  if (!it.fileInfo ().isDir ())
	    {
	      QString fileName = it.filePath ();
	      if (fileName.startsWith (it.path ()))
		fileName.remove (0, it.path ().length () + 1);
	      if (fileName != NULL
		  && !fileName.contains ("MUTE_incoming", Qt::CaseInsensitive)
		  && !fileName.contains ("MUTE_INTERNAL",
					 Qt::CaseInsensitive))
		{
		  int indfic = findFilename (fileName.toLocal8Bit ().data ());
		  if (indfic < 0
		      || Shares[indfic].size != it.fileInfo ().size ()
		      || it.fileInfo ().lastModified () !=
		      *Shares[indfic].time)
		    {
		      printf ("hashing %s\n",
			      fileName.toLocal8Bit ().data ());
		      // generate a new hash
		      QFile ficin (it.filePath ());
		      QCryptographicHash hash (QCryptographicHash::Sha1);
		      ficin.open (QIODevice::ReadOnly);
		      QByteArray buf;
		      do
			{
			  buf = ficin.read (10240);
			  hash.addData (buf);
			  //limit CPU usage by sleeping when more than 10 ms elapsed
			  if (time0.elapsed () > 10)
			    {
			      usleep ((time0.elapsed () * (100 - percentCPU) *
				       1000) / percentCPU);
			      time0 = QTime::currentTime ();
			    }
			}
		      while (buf.size () == 10240 && running);

		      if (running)
			{
			  printf ("hashed %s\n",
				  fileName.toLocal8Bit ().data ());
			  // cache it
			  Shares[nbShares].name =
			    strdup (fileName.toUtf8 ().data ());
			  Shares[nbShares].hash =
			    strdup (hash.result ().toHex ().toUpper ().
				    data ());
			  QFileInfo ficinfo = QFileInfo (ficin);
			  Shares[nbShares].time =
			    new QDateTime (ficinfo.lastModified ());
			  Shares[nbShares++].size = ficin.size ();
			  doPost (&Shares[nbShares - 1]);
			  if (nbShares >= maxShares)
			    {
			      maxShares += 1000;
			      Shares =
				(SHASH *) realloc (Shares,
						   (maxShares +
						    1) * sizeof (SHASH));
			    }
			}
		    }
		}
	    }
	}
    }
}
