/***************************************************************************
 *   Copyright (C) 2006 by Niels Sandholt Busch                            *
 *   niels@diku.dk                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QList>
#include <QtDebug>
#include <QContextMenuEvent>
#include <QMenu>
#include <QVBoxLayout>
#include <QIcon>
#include <QStandardItemModel>
#include <QModelIndex>
#include <QHeaderView>
#include <QDir>
#include <QFileInfo>

#include "kommute.h"
#include "kommutedefs.h"
#include "connectionevent.h"
#include "connectioncountevent.h"
#include "connectionattemptevent.h"
#include "download.h"
#include "downloadstable.h"
#include "DLListDelegate.h"
#include "ConnectionsListDelegate.h"
#include "SFListDelegate.h"
#include "sharedfilesevent.h"
#include "properities/DetailsDialog.h"

#include "uploadevent.h"
#include "uploadscountevent.h"
#include "uploadlistitem.h"
#include "uploadslist.h"
#include "kommuteutils.h"
#include "mutelayer.h"

void Kommute::initMultiview()
{

    ConnectionsListModel = new QStandardItemModel(0, 6);
    ConnectionsListModel->setHeaderData(CICON, Qt::Horizontal, tr(""));
    ConnectionsListModel->setHeaderData(ADDRESS, Qt::Horizontal, tr("Address"));
    ConnectionsListModel->setHeaderData(PORT, Qt::Horizontal, tr("Port"));
    ConnectionsListModel->setHeaderData(SENT, Qt::Horizontal, tr("Sent"));
    ConnectionsListModel->setHeaderData(QUEUED, Qt::Horizontal, tr("Queued"));
    ConnectionsListModel->setHeaderData(DROPPED, Qt::Horizontal, tr("Dropped"));
    
    connectionsTreeView->setModel(ConnectionsListModel);
    CLDelegate = new ConnectionsListDelegate(connectionsTreeView);
    connectionsTreeView->setItemDelegate(CLDelegate);
    
    connectionsTreeView->setSortingEnabled(true);
    connectionsTreeView->setRootIsDecorated(false);
    /* Set header resize modes and initial section sizes */
	QHeaderView * _cheader = connectionsTreeView->header();
	_cheader->setResizeMode ( 0, QHeaderView::Custom);
    _cheader->resizeSection ( 0, 20 );
	_cheader->resizeSection ( 1, 100 );
	_cheader->resizeSection ( 2, 100 );
	_cheader->resizeSection ( 3, 100 );
	_cheader->resizeSection ( 4, 100 );
	_cheader->resizeSection ( 5, 100 );

    DLListModel = new QStandardItemModel(0, 15);
    DLListModel->setHeaderData(SYMBOL, Qt::Horizontal, tr("#"));
    DLListModel->setHeaderData(NAME, Qt::Horizontal, tr("File Name"));
    DLListModel->setHeaderData(SIZE, Qt::Horizontal, tr("Size"));
    DLListModel->setHeaderData(COMPLETED, Qt::Horizontal, tr("Completed"));
    DLListModel->setHeaderData(DLSPEED, Qt::Horizontal, tr("Speed"));
    DLListModel->setHeaderData(PROGRESS, Qt::Horizontal, tr("Progress"));
    DLListModel->setHeaderData(SOURCES, Qt::Horizontal, tr("Sources"));
    DLListModel->setHeaderData(ROUTEQUALITY, Qt::Horizontal, tr("Route Quality"));    
    DLListModel->setHeaderData(STATUS, Qt::Horizontal, tr("Status"));
    DLListModel->setHeaderData(REMAINING, Qt::Horizontal, tr("Remaining"));
    DLListModel->setHeaderData(DONE, Qt::Horizontal, tr("Done"));
    DLListModel->setHeaderData(HOST, Qt::Horizontal, tr("Host"));
    DLListModel->setHeaderData(CHUNKS, Qt::Horizontal, tr("Chunks"));
    DLListModel->setHeaderData(HASH, Qt::Horizontal, tr("Hash"));
    DLListModel->setHeaderData(PTR, Qt::Horizontal, tr("ptr"));

    downloadsTreeView->setModel(DLListModel);
    DLDelegate = new DLListDelegate(downloadsTreeView);
    downloadsTreeView->setItemDelegate(DLDelegate);
    
    downloadsTreeView->hideColumn(14);
    downloadsTreeView->setSortingEnabled(true);
    downloadsTreeView->setRootIsDecorated(false);
    
     /* Set header resize modes and initial section sizes */
	QHeaderView * _dlheader = downloadsTreeView->header () ;   
	_dlheader->setResizeMode (SYMBOL, QHeaderView::Custom);
	_dlheader->setResizeMode (NAME, QHeaderView::Interactive);
	_dlheader->setResizeMode (SIZE, QHeaderView::Interactive);
	_dlheader->setResizeMode (COMPLETED, QHeaderView::Interactive);
    _dlheader->setResizeMode (DLSPEED, QHeaderView::Interactive);
	_dlheader->setResizeMode (PROGRESS, QHeaderView::Interactive);
	_dlheader->setResizeMode (SOURCES, QHeaderView::Interactive);
    _dlheader->setResizeMode (HASH, QHeaderView::Interactive);
   
	_dlheader->resizeSection ( SYMBOL, 20 );
	_dlheader->resizeSection ( NAME, 200 );
	_dlheader->resizeSection ( SIZE, 60 );
	_dlheader->resizeSection ( COMPLETED, 80 );
    _dlheader->resizeSection ( DLSPEED, 60 );
	_dlheader->resizeSection ( PROGRESS, 160 );
	_dlheader->resizeSection ( SOURCES, 60 );
	_dlheader->resizeSection ( REMAINING, 75 );
    _dlheader->resizeSection ( DONE, 60 );
	_dlheader->resizeSection ( HASH, 270 );
    

    //connect(downloadsTreeView, SIGNAL(downloadCanceled(qint64)), parent, SLOT(cancelDownload(qint64)));
    connect(downloadsTreeView, SIGNAL(grabFriendRequested(QString,ulong,QString,QString)),
	this, SLOT(addFriendToSearchView( QString, unsigned long, QString, QString )));

    connect(uploadsList, SIGNAL(grabFriendRequested(QString, unsigned long, QString, QString )),
	this, SLOT(addFriendToSearchView( QString, unsigned long, QString, QString )));
}

void Kommute::customEvent(QEvent *e)
{
    int c;
    QModelIndex index;

    bool found = false;
    switch(e->type())
    {
        case EVENTTYPE_CLEARCONNECTIONS:
	    c = ConnectionsListModel->rowCount();
	    ConnectionsListModel->removeRows(0,c);
            break;
        case EVENTTYPE_CONNECTION:
            c = ConnectionsListModel->rowCount();
            ConnectionsListModel->insertRow(c);
            index = ConnectionsListModel->index(c, CICON);
            ConnectionsListModel->setData(index, QIcon(QString::fromUtf8(":/resources/contact0.png")), Qt::DecorationRole);
		    index = ConnectionsListModel->index(c, ADDRESS);
            ConnectionsListModel->setData(index, static_cast<ConnectionEvent*>(e)->address);
            index = ConnectionsListModel->index(c, PORT);
            ConnectionsListModel->setData(index, static_cast<ConnectionEvent*>(e)->port);
            index = ConnectionsListModel->index(c, SENT);
            ConnectionsListModel->setData(index, static_cast<ConnectionEvent*>(e)->sentCount);
            index = ConnectionsListModel->index(c, QUEUED);
            ConnectionsListModel->setData(index, static_cast<ConnectionEvent*>(e)->queuedCount);
            index = ConnectionsListModel->index(c, DROPPED);
            ConnectionsListModel->setData(index, static_cast<ConnectionEvent*>(e)->droppedCount);
            break;
        case EVENTTYPE_CONNECTIONCOUNT:
            emit connectionCountChanged(static_cast<ConnectionCountEvent*>(e)->connectionCount);
            break;
        case EVENTTYPE_CONNECTIONATTEMPT:
            emit currentConnectionAttempt(static_cast<ConnectionAttemptEvent*>(e)->attemptAddress,
                                          static_cast<ConnectionAttemptEvent*>(e)->attemptPort );
            break;
        case EVENTTYPE_UPLOAD:
            for (QTreeWidgetItemIterator it(uploadsList); *it; ++it )
            {
                if(static_cast<UploadEvent*>(e)->uploadID == static_cast<UploadListItem*>(*it)->uploadID())
                {
                    // Update existing upload
                    found = true;
                    static_cast<UploadListItem*>(*it)->setProgress(static_cast<UploadEvent*>(e)->lastChunkSent,
                                                                       static_cast<UploadEvent*>(e)->chunksInFile,
                                                                       static_cast<UploadEvent*>(e)->status );
                    break;
                }
            }

            if(!found)
            {
                // Create new upload entry
                new UploadListItem( uploadsList,
                                    static_cast<UploadEvent*>(e)->filePath,
                                    static_cast<UploadEvent*>(e)->host,
                                    static_cast<UploadEvent*>(e)->uploadID );
            }

            break;
        case EVENTTYPE_UPLOADSCOUNT:
            emit numberOfUploadsChanged( static_cast<UploadsCountEvent*>(e)->uploadsCount );
            break;
        case EVENTTYPE_SHAREDFILES:
            c = SFListModel->rowCount();
            SFListModel->insertRow(c);
            index = SFListModel->index(c, SFNAME);
            SFListModel->setData(index, static_cast<SharedFilesEvent*>(e)->FilePath);
            index = SFListModel->index(c, SFSIZE);
            SFListModel->setData(index, static_cast<unsigned long long int>(static_cast<SharedFilesEvent*>(e)->FileLength));
            index = SFListModel->index(c, SFHASH);
            SFListModel->setData(index, static_cast<SharedFilesEvent*>(e)->FileHash);
            break;
        default:
            break;
    }

}

void Kommute::downloadFileRequested(QString fname,
                                      unsigned long fsize,
                                      QString fhash,
                                      QString fhost)
{
    for (int i =0 ; i < DLListModel->rowCount() ; i++)
    {
	if(DLListModel->data(DLListModel->index(i,HASH)).toString()==fhash)
	{
	    fprintf(stderr,"file is already in download list\n");
	    //TODO
	    return;
	}
    }
    Download *dl = new Download( static_cast<QStandardItemModel*>(DLListModel),
                   fname,
                   fsize,
                   fhash,
                   fhost );

    dl->start();


}

//void Kommute::hideEvent( QHideEvent *e )
//{
 //   emit visible(false);
//}

// void MultiView::cancelDownload( qint64 id )
// {
//     Download *dl = dlMap[id];
//     dl->cancel();
//     dl->delHash();
// }
char *DEFAULT_INCOMINGHASH_DIR ="MUTE_incoming_hashes";

void Kommute::reloadList()
{
    /* TODO : load list of previous downloads */
    char *sharePathString = MuteLayer::muteLayerGetSharingPath();
    QString fullPath( sharePathString );
    delete [] sharePathString;
    fullPath += "/";
    fullPath += DEFAULT_INCOMINGHASH_DIR;

    QDir dir;
    dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
    dir.setSorting(QDir::Time );
    if (!dir.cd(fullPath))
    {
      // create dir if not exists
      dir.mkdir(fullPath);
      return;
    }
    QFileInfoList list= dir.entryInfoList();
    QFileInfo fileInfo;
    for ( int i=0 ; i< list.size() ; ++i )
    {
       fileInfo = list.at(i);
       QFile FileHash(fileInfo.absoluteFilePath());
       if ( !FileHash.open( QIODevice::ReadOnly))
       {
	       fprintf(stderr,"can't open file %s\n",fileInfo.absoluteFilePath().toLocal8Bit().data());
            return ;
       }
       QTextStream Stream ( & FileHash );
       QString fname,s_fsize,fhost;
       fname=Stream.readLine();
       s_fsize=Stream.readLine();
       fhost=Stream.readLine();

    unsigned long fsize=s_fsize.toULong();
       downloadFileRequested(fname,
		          fsize,
                          fileInfo.fileName(),
			  fhost
			  );
    }
}

void Kommute::updateProgress(int value)
{
	for(int i = 0; i <= DLListModel->rowCount(); i++) {
		if(selection->isRowSelected(i, QModelIndex())) {
			editItem(i, PROGRESS, QVariant((double)value));
		}
	}
}

int Kommute::addItem(QString symbol, QString name, qlonglong fileSize, double progress, double dlspeed, QString sources,  QString status, QString routequality, QString host, QString hash, QString ptr, qlonglong completed, qlonglong remaining)
{
	int row;
	QString sl;
	//QIcon icon(symbol);
	name.insert(0, " ");
	//sl.sprintf("%d / %d", seeds, leechs);
	row = DLListModel->rowCount();
	DLListModel->insertRow(row);

	//DLListModel->setData(DLListModel->index(row, SYMBOL), QVariant((QIcon)icon), Qt::DecorationRole);
	DLListModel->setData(DLListModel->index(row, NAME), QVariant((QString)name), Qt::DisplayRole);
	DLListModel->setData(DLListModel->index(row, SIZE), QVariant((qlonglong)fileSize));
	DLListModel->setData(DLListModel->index(row, PROGRESS), QVariant((double)progress));
	DLListModel->setData(DLListModel->index(row, DLSPEED), QVariant((double)dlspeed));
	DLListModel->setData(DLListModel->index(row, SOURCES), QVariant((QString)sources));
	DLListModel->setData(DLListModel->index(row, STATUS), QVariant((QString)status));
    DLListModel->setData(DLListModel->index(row, ROUTEQUALITY), QVariant((QString)routequality));
    DLListModel->setData(DLListModel->index(row, HOST), QVariant((QString)host));
    DLListModel->setData(DLListModel->index(row, HASH), QVariant((QString)hash));
    DLListModel->setData(DLListModel->index(row, PTR), QVariant((QString)ptr));
	DLListModel->setData(DLListModel->index(row, COMPLETED), QVariant((qlonglong)completed));
	DLListModel->setData(DLListModel->index(row, REMAINING), QVariant((qlonglong)remaining));
	return row;
}

void Kommute::delItem(int row)
{
	DLListModel->removeRow(row, QModelIndex());
}

void Kommute::editItem(int row, int column, QVariant data)
{
	//QIcon *icon;
	switch(column) {
		//case SYMBOL:
			/*icon = new QIcon(data.toString());*/
			//DLListModel->setData(DLListModel->index(row, SYMBOL), QVariant((QIcon)*icon), Qt::DecorationRole);
			//delete icon;
			//break;
		case NAME:
			DLListModel->setData(DLListModel->index(row, NAME), data, Qt::DisplayRole);
			break;
		case SIZE:
			DLListModel->setData(DLListModel->index(row, SIZE), data);
			break;
		case PROGRESS:
			DLListModel->setData(DLListModel->index(row, PROGRESS), data);
			break;
		case DLSPEED:
			DLListModel->setData(DLListModel->index(row, DLSPEED), data);
			break;
		case SOURCES:
			DLListModel->setData(DLListModel->index(row, SOURCES), data);
			break;
		case STATUS:
			DLListModel->setData(DLListModel->index(row, STATUS), data);
			break;
	    case ROUTEQUALITY:
			DLListModel->setData(DLListModel->index(row, ROUTEQUALITY), data);
			break;
		case HOST:
			DLListModel->setData(DLListModel->index(row, HOST), data);
			break;
		case HASH:
			DLListModel->setData(DLListModel->index(row, HASH), data);
			break;
		case PTR:
			DLListModel->setData(DLListModel->index(row, PTR), data);
			break;
		case COMPLETED:
			DLListModel->setData(DLListModel->index(row, COMPLETED), data);
			break;
		case REMAINING:
			DLListModel->setData(DLListModel->index(row, REMAINING), data);
			break;
	}
}

double Kommute::getProgress(int row, QStandardItemModel *model)
{
	return model->data(model->index(row, PROGRESS), Qt::DisplayRole).toDouble();
}

double Kommute::getSpeed(int row, QStandardItemModel *model)
{
	return model->data(model->index(row, DLSPEED), Qt::DisplayRole).toDouble();
}

QString Kommute::getFileName(int row, QStandardItemModel *model)
{
	return model->data(model->index(row, NAME), Qt::DisplayRole).toString();
}

QString Kommute::getStatus(int row, QStandardItemModel *model)
{
	return model->data(model->index(row, STATUS), Qt::DisplayRole).toString();
}

qlonglong Kommute::getFileSize(int row, QStandardItemModel *model)
{
	bool ok = false;
	return model->data(model->index(row, SIZE), Qt::DisplayRole).toULongLong(&ok);
}

QString Kommute::getRouteQuality(int row, QStandardItemModel *model)
{
	return model->data(model->index(row, ROUTEQUALITY), Qt::DisplayRole).toString();
}

QString Kommute::getHost(int row, QStandardItemModel *model)
{
	return model->data(model->index(row, HOST), Qt::DisplayRole).toString();
}

QString Kommute::getHash(int row, QStandardItemModel *model)
{
	return model->data(model->index(row, HASH), Qt::DisplayRole).toString();
}

QString Kommute::getPtr(int row, QStandardItemModel *model)
{
	return model->data(model->index(row, PTR), Qt::DisplayRole).toString();
}

qlonglong Kommute::getTransfered(int row, QStandardItemModel *model)
{
	bool ok = false;
	return model->data(model->index(row, COMPLETED), Qt::DisplayRole).toULongLong(&ok);
}

qlonglong Kommute::getRemainingTime(int row, QStandardItemModel *model)
{
	bool ok = false;
	return model->data(model->index(row, REMAINING), Qt::DisplayRole).toULongLong(&ok);
}


int Kommute::addCLItem(QString cicon, QString address, QString port, qlonglong sent, QString queued, QString dropped )
{
	int row;
	QString sl;
	//QIcon icon(cicon);
	name.insert(0, " ");
	//sl.sprintf("%d / %d", seeds, leechs);
	row = ConnectionsListModel->rowCount();
	ConnectionsListModel->insertRow(row);

	//ConnectionsListModel->setData(ConnectionsListModel->index(row, CICON), QVariant((QIcon)icon), Qt::DecorationRole);
	ConnectionsListModel->setData(ConnectionsListModel->index(row, ADDRESS), QVariant((QString)address));
    ConnectionsListModel->setData(ConnectionsListModel->index(row, PORT), QVariant((QString)port));
	ConnectionsListModel->setData(ConnectionsListModel->index(row, SENT), QVariant((qlonglong)sent));
	ConnectionsListModel->setData(ConnectionsListModel->index(row, QUEUED), QVariant((QString)queued));
	ConnectionsListModel->setData(ConnectionsListModel->index(row, DROPPED), QVariant((QString)dropped));
	return row;
}

void Kommute::delCLItem(int row)
{
	ConnectionsListModel->removeRow(row, QModelIndex());
}

void Kommute::editCLItem(int row, int column, QVariant data)
{
	//QIcon *icon;
	switch(column) {
		//case CICON:
			/*icon = new QIcon(data.toString());*/
			//ConnectionsListModel->setData(ConnectionsListModel->index(row, SYMBOL), QVariant((QIcon)*icon), Qt::DecorationRole);
			//delete icon;
			//break;
		case ADDRESS:
			ConnectionsListModel->setData(ConnectionsListModel->index(row, ADDRESS), data);
			break;
		case PORT:
			ConnectionsListModel->setData(ConnectionsListModel->index(row, PORT), data);
			break;
		case SENT:
			ConnectionsListModel->setData(ConnectionsListModel->index(row, SENT), data);
			break;
		case QUEUED:
			ConnectionsListModel->setData(ConnectionsListModel->index(row, QUEUED), data);
			break;
		case DROPPED:
			ConnectionsListModel->setData(ConnectionsListModel->index(row, DROPPED), data);
			break;
	}
}

// display Details of selected items
void Kommute::detailsSelection(){
  QModelIndexList selectedIndexes = downloadsTreeView->selectionModel()->selectedIndexes();
  QModelIndex index;
  foreach(index, selectedIndexes){
    if(index.column() == NAME){
      showDetails(index);
    }
  }
}

// Show File Details dialog
void Kommute::showDetails(const QModelIndex &index) {
  int row = index.row();
  QString hash = DLListModel->data(DLListModel->index(row, HASH)).toString();
  DetailsDialog *details = new DetailsDialog(this);
  //connect(prop, SIGNAL(filteredFilesChanged(QString)), this, SLOT(updateFileSizeAndProgress(QString)));
  details->show();
}
/*
void Kommute::updateFileSizeAndProgress(QString hash) {
  int row = getRowFromHash(hash);
  //Q_ASSERT(row != -1);
  DLListModel->setData(DLListModel->index(row, SIZE), QVariant((qlonglong)fileSize()));
  DLListModel->setData(DLListModel->index(row, PROGRESS), QVariant((double)progress()));
}*/

// return the row of in data model
int Kommute::getRowFromHash(QString hash) const{
  unsigned int nbRows = DLListModel->rowCount();
  for(unsigned int i=0; i<nbRows; ++i) {
    if(DLListModel->data(DLListModel->index(i, HASH)) == hash) {
      return i;
    }
  }
  return -1;
}

