/***************************************************************************
 *   Copyright (C) 2006 by Niels Sandholt Busch                            *
 *   niels@diku.dk                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "searchresultstable.h"
#include "searchresultsview.h"

#include <QContextMenuEvent>
#include <QMenu>
#include <QAction>
#include <QTableView>
#include <QTreeView>
#include <QModelIndex>
#include <QAbstractTableModel>
#include <QVariant>
#include <QString>
#include <QHeaderView>

SearchResultsTable::SearchResultsTable (QWidget * parent)
:QTreeView (parent)
{
  //verticalHeader ()->hide ();

}

SearchResultsTable::~SearchResultsTable ()
{
}

void
SearchResultsTable::contextMenuEvent (QContextMenuEvent * e)
{
  QMenu popMenu (this);
  QAction *actionDownload = popMenu.addAction (tr ("Download"));
  actionDownload->setIcon (QIcon (":/resources/start.png"));
  connect (actionDownload, SIGNAL (triggered ()), this, SLOT (download ()));
  
  popMenu.addSeparator ();
  
  QAction *actionGrab = popMenu.addAction (tr ("Grab Friend"));
  actionGrab->setIcon (QIcon (":/grabfriend.png"));
  connect (actionGrab, SIGNAL (triggered ()), this, SLOT (grabFriend ()));
  
  popMenu.setMouseTracking (true);
  popMenu.exec (QCursor::pos ());
}

void
SearchResultsTable::grabFriend ()
{
  QModelIndexList lst = selectedIndexes ();
  QModelIndex & ind = lst[0];
  for (int i = 0; i < lst.count (); i++)
    {
      if (lst[i].column () == 0)
	{
	  ind = lst[i];
	  QString fname = ind.sibling (ind.row (), SEARCH_NAME).data ().toString ();
	  unsigned long fsize =
	    static_cast <
	    unsigned long >(ind.sibling (ind.row (), SEARCH_SIZE).data ().
			    toULongLong ());
	  QString fhash = ind.sibling (ind.row (), SEARCH_HASH).data ().toString ();
	  QString fhost = ind.sibling (ind.row (), SEARCH_HOSTHASH).data ().toString ();
	  emit grabFriendRequested (fname, fsize, fhash, fhost);
	}
    }
}
void
SearchResultsTable::download ()
{
  QModelIndexList lst = selectedIndexes ();
  QModelIndex & ind = lst[0];
  for (int i = 0; i < lst.count (); i++)
    {
      if (lst[i].column () == 0)
	{
	  ind = lst[i];
	  QString fname = ind.sibling (ind.row (), SEARCH_NAME).data ().toString ();
	  unsigned long fsize =
	    static_cast <
	    unsigned long >(ind.sibling (ind.row (), SEARCH_SIZE).data ().
			    toULongLong ());
	  QString fhash = ind.sibling (ind.row (), SEARCH_HASH).data ().toString ();
	  QString fhost = ind.sibling (ind.row (), SEARCH_HOSTHASH).data ().toString ();
	  emit downloadFileRequested (fname, fsize, fhash, fhost);
	}
    }
}
