/***************************************************************************
 *   Copyright (C) 2006 by Niels Sandholt Busch                            *
 *   niels@diku.dk                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "searchresultsview.h"
#include "searchresultevent.h"
#include "SearchListDelegate.h"
#include "closesearchthreadqueue.h"
#include "kommuteutils.h"
#include "kommutedefs.h"
#include "mutelayer.h"
#include "downloadstable.h"
#include "download.h"
#include "DLListDelegate.h"

#include <QSettings>
#include <QStandardItemModel>
#include <QModelIndex>
#include <QMenu>
#include <QAction>
#include <QContextMenuEvent>
#include <QHeaderView>

SearchResultsView::SearchResultsView(QWidget *parent, QString address, QString searchTerms, CloseSearchThreadQueue *searchThreadQueue)
 : QWidget(parent),
   address( address ),
   searchTerms( searchTerms ),
   searchThreadQueue( searchThreadQueue ),
   active( true ),
   numSearchResults( 0 )
{
    setupUi(this);

    SearchListModel = new QStandardItemModel(0, (SEARCH_LASTCOLUMN+1));

    SearchListModel->setHeaderData(SEARCH_NAME, Qt::Horizontal, tr("File Name"));
    SearchListModel->setHeaderData(SEARCH_SIZE, Qt::Horizontal, tr("Size"));
    SearchListModel->setHeaderData(SEARCH_HASH, Qt::Horizontal, tr("Hash"));
    SearchListModel->setHeaderData(SEARCH_HOST, Qt::Horizontal, tr("Host"));
//    SearchListModel->setHeaderData(SEARCH_SIZEB, Qt::Horizontal, tr("Size (b)"));
    SearchListModel->setHeaderData(SEARCH_HOSTHASH, Qt::Horizontal, tr("Host (hash)"));
    proxyModel = new QSortFilterProxyModel;
    proxyModel->setDynamicSortFilter(true);
    proxyModel->setSourceModel(SearchListModel);
    searchtreeView->setModel(proxyModel);
    
    SearchDelegate = new SearchListDelegate();
    searchtreeView->setItemDelegate(SearchDelegate);
    
    searchtreeView->setSortingEnabled(true);
    searchtreeView->setDragDropOverwriteMode(false);
    searchtreeView->setDropIndicatorShown(false);
    searchtreeView->setTabKeyNavigation(false);
    /* hide the Tree +/- */
    searchtreeView->setRootIsDecorated(false);
    
    //searchtreeView->header()->setClickable(true);
    //searchtreeView->header()->setSortIndicatorShown(true);
    // Load last columns width for search results list
    if(!loadColWidthSearchList()){
     searchtreeView->header()->resizeSection(SEARCH_NAME, 275);
	}
    
    /* Set header resize modes and initial section sizes */
	QHeaderView * _header =  searchtreeView->header () ;   
	_header->setResizeMode (SEARCH_SIZE, QHeaderView::Interactive);
	_header->setResizeMode (SEARCH_HASH, QHeaderView::Interactive);
	_header->setResizeMode (SEARCH_HOST, QHeaderView::Interactive);
//	_header->setResizeMode (SEARCH_SIZEB, QHeaderView::Interactive);
	_header->setResizeMode (SEARCH_HOSTHASH, QHeaderView::Interactive);
	_header->resizeSection ( SEARCH_SIZE, 70 );
	_header->resizeSection ( SEARCH_HASH, 260 );
	_header->resizeSection ( SEARCH_HOST, 80 );
//	_header->resizeSection ( SEARCH_SIZEB, 70 );
	_header->resizeSection ( SEARCH_HOSTHASH, 260 );
	
	
	//connect(searchtreeView->header(), SIGNAL(sectionPressed(int)), this, SLOT(sortSearchList(int)));
    connect(searchtreeView, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(downloadSelectedItem(const QModelIndex&)));

    connect(searchtreeView, SIGNAL(downloadFileRequested(QString, unsigned long, QString, QString)),
                 this, SLOT(download(QString, unsigned long, QString, QString)));
    connect(searchtreeView, SIGNAL(grabFriendRequested(QString, unsigned long, QString, QString )),
	this, SLOT(addFriend( QString, unsigned long, QString, QString )));
    connect(searchLine, SIGNAL(textChanged(const QString &)),
		                 this, SLOT(filterRegExpChanged()));
}

SearchResultsView::~SearchResultsView()
{
    sThread->eventsAllowed( false );
    active = false;
    searchThreadQueue->insertSearchThread( sThread );
    
    saveColWidthSearchList();
    delete SearchListModel;
    delete SearchDelegate;
}

void SearchResultsView::on_clearfilter_Button_clicked()
{
    searchLine->clear();
}

void SearchResultsView::filterRegExpChanged()
{
    QRegExp regExp(searchLine->text(), Qt::CaseInsensitive);
    proxyModel->setFilterRegExp(regExp);
    emit numberOfSearchResultsChanged( this, numSearchResults, proxyModel->rowCount() );
}

void SearchResultsView::attachSearchThread( SearchThread *sThread )
{
    this->sThread = sThread;
}

void SearchResultsView::customEvent( QEvent *e )
{
    int c;
    QModelIndex index;

    switch( e->type() ) {
        case EVENTTYPE_SEARCHRESULT:
            c = SearchListModel->rowCount();
            SearchListModel->insertRow(c);
            index = SearchListModel->index(c, SEARCH_NAME);
            SearchListModel->setData(index, static_cast<SearchResultEvent*>(e)->inFilePath);
            index = SearchListModel->index(c, SEARCH_SIZE);
            SearchListModel->setData(index, static_cast<unsigned long long int>(static_cast<SearchResultEvent*>(e)->inFileLength));
            index = SearchListModel->index(c, SEARCH_HASH);
            SearchListModel->setData(index, static_cast<SearchResultEvent*>(e)->inFileHash);
            index = SearchListModel->index(c, SEARCH_HOST);
            SearchListModel->setData(index, MuteLayer::getMnemonic(static_cast<SearchResultEvent*>(e)->inFileHostVirtualAddress));
//            index = SearchListModel->index(c, SEARCH_SIZEB);
//            SearchListModel->setData(index, KommuteUtils::convertSize(static_cast<SearchResultEvent*>(e)->inFileLength));

            index = SearchListModel->index(c, SEARCH_HOSTHASH);
            SearchListModel->setData(index, (static_cast<SearchResultEvent*>(e)->inFileHostVirtualAddress));

            numSearchResults++;
            emit numberOfSearchResultsChanged( this, numSearchResults, proxyModel->rowCount() );
            break;
        default:
            break;
    }
}

void SearchResultsView::download(QString fname,
                                 unsigned long fsize,
                                 QString fhash,
                                 QString fhost)
{
    emit downloadFileRequested(fname, fsize, fhash, fhost);
}

QString SearchResultsView::getSearchTerms()
{
    return searchTerms;
}
void SearchResultsView::addFriend(QString fname,
		unsigned long fsize,
		QString fhash,
		QString fhost)
{
    emit grabFriendRequested(fname, fsize, fhash, fhost);
}

// Download the given item from search results list
void SearchResultsView::downloadSelectedItem(const QModelIndex& index){
  int row = index.row();


  // Set item color to RED
  setRowColor(row, "red");
}


// Set the color of a row in data model
void SearchResultsView::setRowColor(int row, QString color){
  for(int i=0; i<SearchListModel->columnCount(); ++i){
    SearchListModel->setData(SearchListModel->index(row, i), QVariant(QColor(color)), Qt::ForegroundRole);
  }
}

// Sort the Search List
void SearchResultsView::sortSearchList(int index){
  static Qt::SortOrder sortOrder = Qt::AscendingOrder;
  if(searchtreeView->header()->sortIndicatorSection() == index){
    if(sortOrder == Qt::AscendingOrder){
      sortOrder = Qt::DescendingOrder;
    }else{
      sortOrder = Qt::AscendingOrder;
    }
  }
  searchtreeView->header()->setSortIndicator(index, sortOrder);
  switch(index){
    //case SIZE:
    //case SOURCES:
    case SIZE:
      sortSearchListInt(index, sortOrder);
      break;
    default:
      sortSearchListString(index, sortOrder);
  }
}


void SearchResultsView::sortSearchListInt(int index, Qt::SortOrder sortOrder){
  QList<QPair<int, qlonglong> > lines;
  // Insertion sorting
  for(int i=0; i<SearchListModel->rowCount(); ++i){
    misc::insertSort(lines, QPair<int,qlonglong>(i, SearchListModel->data(SearchListModel->index(i, index)).toLongLong()), sortOrder);
  }
  // Insert items in new model, in correct order
  int nbRows_old = lines.size();
  for(int row=0; row<lines.size(); ++row){
    SearchListModel->insertRow(SearchListModel->rowCount());
    int sourceRow = lines[row].first;
    for(int col=0; col<=SEARCH_LASTCOLUMN; ++col){
      SearchListModel->setData(SearchListModel->index(nbRows_old+row, col), SearchListModel->data(SearchListModel->index(sourceRow, col)));
      SearchListModel->setData(SearchListModel->index(nbRows_old+row, col), SearchListModel->data(SearchListModel->index(sourceRow, col), Qt::ForegroundRole), Qt::ForegroundRole);
    }
  }
  // Remove old rows
  SearchListModel->removeRows(0, nbRows_old);
}


void SearchResultsView::sortSearchListString(int index, Qt::SortOrder sortOrder){
  QList<QPair<int, QString> > lines;
  // Insetion sorting
  for(int i=0; i<SearchListModel->rowCount(); ++i){
    misc::insertSortString(lines, QPair<int, QString>(i, SearchListModel->data(SearchListModel->index(i, index)).toString()), sortOrder);
  }
  // Insert items in new model, in correct order
  int nbRows_old = lines.size();
  for(int row=0; row<nbRows_old; ++row){
    SearchListModel->insertRow(SearchListModel->rowCount());
    int sourceRow = lines[row].first;
    for(int col=0; col<=SEARCH_LASTCOLUMN; ++col){
      SearchListModel->setData(SearchListModel->index(nbRows_old+row, col), SearchListModel->data(SearchListModel->index(sourceRow, col)));
      SearchListModel->setData(SearchListModel->index(nbRows_old+row, col), SearchListModel->data(SearchListModel->index(sourceRow, col), Qt::ForegroundRole), Qt::ForegroundRole);
    }
  }
  // Remove old rows
  SearchListModel->removeRows(0, nbRows_old);
}

// Save columns width in a file to remember them
// (download list)
void SearchResultsView::saveColWidthSearchList() const{
  qDebug("Saving columns width in search list");
  QSettings settings("Kommute", "Kommute");
  QStringList width_list;
  for(int i=0; i<SearchListModel->columnCount(); ++i){
    width_list << misc::toQString(searchtreeView->columnWidth(i));
  }
  settings.setValue("SearchListColsWidth", width_list.join(" "));
  qDebug("Search list columns width saved");
}


// Load columns width in a file that were saved previously
// (search list)
bool SearchResultsView::loadColWidthSearchList(){
  qDebug("Loading columns width for search list");
  QSettings settings("Kommute", "Kommute");
  QString line = settings.value("SearchListColsWidth", QString()).toString();
  if(line.isEmpty())
    return false;
  QStringList width_list = line.split(' ');
  if(width_list.size() != SearchListModel->columnCount())
    return false;
  for(int i=0; i<width_list.size(); ++i){
        searchtreeView->header()->resizeSection(i, width_list.at(i).toInt());
  }
  qDebug("Search list columns width loaded");
  return true;
}
