/***************************************************************************
 *   Copyright (C) 2005 by Niels Sandholt Busch   *
 *   niels@diku.dk   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "searchthread.h"
#include "searchresultevent.h"

#include "mutelayer.h"

#include <QApplication>
#include <QString>
#include <QTextCodec>

const int   DEFAULT_SEARCHTHREAD_SLEEP = 2;

SearchThread::SearchThread( QObject *eventReceiver, QString address, QString searchTerms )
 : QThread(), eventReceiver( eventReceiver ), running( false ), address( address ), searchTerms( searchTerms ), events( true )
{
}


SearchThread::~SearchThread()
{
    wait();
}

void SearchThread::run()
{
    running = true;

    MuteLayer::muteLayerSearch( (char *) this->searchTerms.toLatin1().constData(),
                     SearchThread::muteSearchHandler,
                     this,1000,( char* ) this->address.toLatin1().constData());

}

void SearchThread::stop()
{
    running = false;
}

char SearchThread::searchHandler( char *inFileHostVirtualAddress, 
                                  char *inFilePath,
                                  unsigned long inFileLength, 
                                  char *inFileHash ) {

    if(inFilePath != NULL)
    {
        if(events) // only post events if eventReceiver has not been deleted
        {
            QTextCodec *codecutf=QTextCodec::codecForName("utf8");
            QTextCodec *codeclat=QTextCodec::codecForName("iso8859-1");
            QString qstr;
	    // try to convert string from utf8
            qstr=codecutf->toUnicode(inFilePath);
	    // test if conversion is reversible
            if(strcmp(inFilePath,qstr.toUtf8().data()))
	      // if not reversible, it's not utf8, convert from iso8859 :
              qstr=codeclat->toUnicode(inFilePath);
            QApplication::postEvent( eventReceiver,
                                     new SearchResultEvent(qstr,
                                                       inFileLength,
                                                       QString::fromLatin1(inFileHash),
                                                       QString::fromLatin1(
                                                                inFileHostVirtualAddress)));
        }
    }

    return running;
}

char SearchThread::muteSearchHandler( char *inFileHostVirtualAddress,
                                             char *inFilePath,
                                             unsigned long inFileLength,
                                             char *inFileHash,
                                             void *inExtraArgument ) {

    return static_cast<SearchThread*>(inExtraArgument)->searchHandler( inFileHostVirtualAddress,
                                                            inFilePath,
                                                            inFileLength,
                                                            inFileHash );
}

void SearchThread::eventsAllowed( bool allowed )
{
    events = allowed;
}
